/**
 * ESP32 version of ultrasonic sensor example.
 * This sketch sends the measured distance to the website https://home.et.utwente.nl/val/dist/
 * Read more about this further down at the doRequest() function.
 * 
 * Libraries used for this sketch:
 * 
 * Ultrasonic sensor uses NewPing library
 * https://bitbucket.org/teckel12/arduino-new-ping/wiki/Home
 * 
 * Display uses U8g2lib
 * https://github.com/olikraus/u8g2
 * 
 * Wifi connectivity realized by WiFiWebClient library
 * https://github.com/tomerweller/esp32-arduino-webclient/blob/master/esp32-webclient.ino
 * 
 * You need to install the ESP32 Arduino Core:
 * https://github.com/espressif/arduino-esp32
 * https://learn.sparkfun.com/tutorials/esp32-thing-hookup-guide#installing-the-esp32-arduino-core
 * 
 * HTTPS:
 * https://techtutorialsx.com/2017/11/18/esp32-arduino-https-get-request/
 * https://github.com/espressif/arduino-esp32/blob/master/libraries/WiFiClientSecure/examples/WiFiClientSecure/WiFiClientSecure.ino
 * 
 * get the right certificate code:
 * https://medium.com/@menakajain/export-download-ssl-certificate-from-server-site-url-bcfc41ea46a2
 * or with this command on for example an Ubuntu webserver:
 * openssl s_client -showcerts -connect home.et.utwente.nl:443 > cert.output
 */

#include <SPI.h>
#include <WiFi.h>
#include <WiFiClientSecure.h>
#include <NewPing.h>
#include <U8g2lib.h>

#define TRIGGER_PIN  4  // Arduino pin tied to trigger pin on the ultrasonic sensor.
#define ECHO_PIN     5  // Arduino pin tied to echo pin on the ultrasonic sensor.
#define MAX_DISTANCE 300 // Maximum distance we want to ping for (in centimeters). Maximum sensor distance is rated at 400-500cm.

NewPing sonar(TRIGGER_PIN, ECHO_PIN, MAX_DISTANCE); // NewPing setup of pins and maximum distance.

// display setup (specific for esp32):
#define SDA 23
#define SCL 19
U8X8_SSD1306_128X64_NONAME_SW_I2C display(SCL, SDA, U8X8_PIN_NONE);

char buf[10]; // text buffer; to be able to use draw2x2String to show the value

#define LED_BUILTIN 22

// Wifi params:
const char* ssid = "mspot";
const char* password = "workshop";

const char* server = "home.et.utwente.nl";

const char* cert="-----BEGIN CERTIFICATE-----\n" \
"MIIFazCCA1OgAwIBAgIRAIIQz7DSQONZRGPgu2OCiwAwDQYJKoZIhvcNAQELBQAw\n" \
"TzELMAkGA1UEBhMCVVMxKTAnBgNVBAoTIEludGVybmV0IFNlY3VyaXR5IFJlc2Vh\n" \
"cmNoIEdyb3VwMRUwEwYDVQQDEwxJU1JHIFJvb3QgWDEwHhcNMTUwNjA0MTEwNDM4\n" \
"WhcNMzUwNjA0MTEwNDM4WjBPMQswCQYDVQQGEwJVUzEpMCcGA1UEChMgSW50ZXJu\n" \
"ZXQgU2VjdXJpdHkgUmVzZWFyY2ggR3JvdXAxFTATBgNVBAMTDElTUkcgUm9vdCBY\n" \
"MTCCAiIwDQYJKoZIhvcNAQEBBQADggIPADCCAgoCggIBAK3oJHP0FDfzm54rVygc\n" \
"h77ct984kIxuPOZXoHj3dcKi/vVqbvYATyjb3miGbESTtrFj/RQSa78f0uoxmyF+\n" \
"0TM8ukj13Xnfs7j/EvEhmkvBioZxaUpmZmyPfjxwv60pIgbz5MDmgK7iS4+3mX6U\n" \
"A5/TR5d8mUgjU+g4rk8Kb4Mu0UlXjIB0ttov0DiNewNwIRt18jA8+o+u3dpjq+sW\n" \
"T8KOEUt+zwvo/7V3LvSye0rgTBIlDHCNAymg4VMk7BPZ7hm/ELNKjD+Jo2FR3qyH\n" \
"B5T0Y3HsLuJvW5iB4YlcNHlsdu87kGJ55tukmi8mxdAQ4Q7e2RCOFvu396j3x+UC\n" \
"B5iPNgiV5+I3lg02dZ77DnKxHZu8A/lJBdiB3QW0KtZB6awBdpUKD9jf1b0SHzUv\n" \
"KBds0pjBqAlkd25HN7rOrFleaJ1/ctaJxQZBKT5ZPt0m9STJEadao0xAH0ahmbWn\n" \
"OlFuhjuefXKnEgV4We0+UXgVCwOPjdAvBbI+e0ocS3MFEvzG6uBQE3xDk3SzynTn\n" \
"jh8BCNAw1FtxNrQHusEwMFxIt4I7mKZ9YIqioymCzLq9gwQbooMDQaHWBfEbwrbw\n" \
"qHyGO0aoSCqI3Haadr8faqU9GY/rOPNk3sgrDQoo//fb4hVC1CLQJ13hef4Y53CI\n" \
"rU7m2Ys6xt0nUW7/vGT1M0NPAgMBAAGjQjBAMA4GA1UdDwEB/wQEAwIBBjAPBgNV\n" \
"HRMBAf8EBTADAQH/MB0GA1UdDgQWBBR5tFnme7bl5AFzgAiIyBpY9umbbjANBgkq\n" \
"hkiG9w0BAQsFAAOCAgEAVR9YqbyyqFDQDLHYGmkgJykIrGF1XIpu+ILlaS/V9lZL\n" \
"ubhzEFnTIZd+50xx+7LSYK05qAvqFyFWhfFQDlnrzuBZ6brJFe+GnY+EgPbk6ZGQ\n" \
"3BebYhtF8GaV0nxvwuo77x/Py9auJ/GpsMiu/X1+mvoiBOv/2X/qkSsisRcOj/KK\n" \
"NFtY2PwByVS5uCbMiogziUwthDyC3+6WVwW6LLv3xLfHTjuCvjHIInNzktHCgKQ5\n" \
"ORAzI4JMPJ+GslWYHb4phowim57iaztXOoJwTdwJx4nLCgdNbOhdjsnvzqvHu7Ur\n" \
"TkXWStAmzOVyyghqpZXjFaH3pO3JLF+l+/+sKAIuvtd7u+Nxe5AW0wdeRlN8NwdC\n" \
"jNPElpzVmbUq4JUagEiuTDkHzsxHpFKVK7q4+63SM1N95R1NbdWhscdCb+ZAJzVc\n" \
"oyi3B43njTOQ5yOf+1CceWxG1bQVs5ZufpsMljq4Ui0/1lvh+wjChP4kqKOJ2qxq\n" \
"4RgqsahDYVvTH9w7jXbyLeiNdd8XM2w9U/t7y0Ff/9yi0GE44Za4rF2LN9d11TPA\n" \
"mRGunUHBcnWEvgJBQl9nJEiU0Zsnvgc/ubhPgXRR4Xq37Z0j4r7g1SgEEzwxA57d\n" \
"emyPxgcYxn/eR44/KJ4EBs+lVDR3veyJm+kXQ99b21/+jh5Xos1AnX5iItreGCc=\n" \
"-----END CERTIFICATE-----\n";

bool led = true;

unsigned int count = 0; // counts products

WiFiClientSecure client;

// function prototype
String doRequest(const char *variable, const char *value, bool reqDisconnect = false);


void setup() {
  Serial.begin(115200); // start serial port output, check for same speed at Serial Monitor

  // enable onboard led as output:
  pinMode(LED_BUILTIN, OUTPUT);
  digitalWrite(LED_BUILTIN, led ? HIGH : LOW);

  // initialize display:
  display.begin();
  display.setPowerSave(0);

  display.setFont(u8x8_font_pxplusibmcgathin_f);
  display.drawString(0,0,"Ultrasonic");
  display.drawString(0,1,"sensor");
  display.draw2x2String(0,3,"Dist.:");

  // initialize Wifi
  Serial.print("Connecting to: ");
  Serial.println(ssid);

  WiFi.begin(ssid, password);
  
  // attempt to connect to Wifi network:
  while(WiFi.status() != WL_CONNECTED) {
    // Connect to WPA/WPA2 network. Change this line if using open or WEP network:
    delay(500);
    led = !led;
    digitalWrite(LED_BUILTIN, led ? HIGH : LOW);
    Serial.print(".");
  }
  Serial.println("");
  Serial.println("WiFi connected");
  Serial.println("IP address: ");
  Serial.println(WiFi.localIP());
  digitalWrite(LED_BUILTIN, HIGH); // connected, turn LED on
}

void loop() {
  unsigned int distance = sonar.convert_cm(sonar.ping_median(5)); // read distance from ultrasonic sensor (using median of 5 iterations)
  // display in serial monitor:
  Serial.print("Dist.: ");
  Serial.println(distance);

  // display value on the display in a large (2x2) font:
  itoa(distance, buf, 10); // put value into string buffer
  display.draw2x2String(0,6,"    "); // clear line
  display.draw2x2String(0,6,buf);

  // send request to webserver: - only 1 in 20 times (to not overload the webserver)
  count++;
  if (count==20) {
    doRequest("dist", buf);
    count=0;
  }
  
  delay(500); // wait shortly
}


/**
 * The doRequest function can store (set) a value in a variable online. It will do a request like this:
 * https://home.et.utwente.nl/val/dist/12 to store variable "dist" with value "12"
 * in this case, the function call looks like:
 * doRequest("dist", "12")
 * 
 * To read a value, just leave the value parameter empty:
 * String value = doRequest("dist", "")
 * which calls: https://home.et.utwente.nl/val/dist/ (you can check the value in your browser also)
 */
String doRequest(const char *variable, const char *value, bool reqDisconnect) {
  digitalWrite(LED_BUILTIN, LOW);
  if (!client.connected()) {
    Serial.println("\nStarting connection to server...");
    // if you get a connection, report back via serial:
    client.setCACert(cert);
    if (!client.connect(server, 443)) {
      Serial.println("Error, could not connect to server");
      return "";
    }
    Serial.println("connected to server");
  }
  // Make a HTTP request:
  String request = "GET /val/"+String(variable);
  if (strlen(value)>0) request += "/"+String(value);
  request += " HTTP/1.1\r\n";
  request += "Host: "+String(server)+"\r\n";
  request += "User-Agent: esp-idf/1.0 esp32\r\n";
  if (reqDisconnect) request += "Connection: close\r\n";
  request += "\r\n";

  Serial.println( request );

  // send request to server:
  client.print( request );
  digitalWrite(LED_BUILTIN, HIGH);

  delay(10); // give the webserver some time to process the request before banging its door for an answer...

  // If there are incoming bytes available
  // from the server, read them and print 
  String line, lastline[2]; // save last two lines
  lastline[0] = "";
  while (client.available()) {
    //char c = client.read();
    //Serial.write(c);
    if (line.length()>0) {
      lastline[1] = lastline[0];
      lastline[0] = line;
    }
    line = client.readStringUntil('\r');
    line.trim();
    Serial.println(line); // debug: show webserver response
  }
  Serial.println( "request done" );
  return lastline[1]; // return second last line (as last line always contains a '0')
}
